#
#       HBS REDUCTION SUB ROUTINE
#
#       Subtracting dark image: hbsdarksub.cl
#
#       1995/10/17      K.Kawabata
#       1996/03/05      K.Kawabata  Ver.1.00  for CRYOCAM
#       1997/03/16      K.Kawabata  Ver.1.20  Bias Correction
#       1998/04/20      K.Kawabata  Ver.2.00  Unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       1999/03/16      K.Kawabata  Ver.2.20  Rejection of abnormal image
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe CCD (bias jump)
#       2001/04/03      K.Kawabata  Ver.3.10  Pixel defect
#       2002/06/10      modify imreplace 
#       2005/04/07      M.Isogai    Ver.3.20 for iraf v2.12
procedure hbsdarksub

begin

  file fn, dark_fn
  file caldir, fn_def
  string fn_stack[500]
  int n_image
  file fn1, fn2, fnnext
  file darklist

  real dthresh = -500.0 # upper limit of dark-subtracted count for defect pixel
  
  file tmp_list, tmp_fn
  int i, j, x, c, flag_d
  string hbslist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string d_title, d_exptime, d_diaph, d_filter
  string d_slit, d_datatyp
  string shortdiaph, d_shortdiaph
  string listoflist, buf
  file logfn
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int n_list
  int flag_erase # 0:keep old images, 1:erase old images
  int flag_ver  # 0:no display 1:display
  string tmp_hotpix = "hbs.tmp.tbl.hotpix"
  string darklog = "HBS.darksub.log"
  real thres_hot
  int flag_hotpix
  real cnt, cnt1, cnt2, cnt3, cnt4
  real cntthresh = 15.0 # in ADU (*0.5 is threshold of bias jump)
  real lastcnt
  real x0 = 0.0
  real y0 = 0.0
  int x1, y1, x2, y2, x3, y3, x4, y4

  file fn_cmnt
  string cmnt
  string cmntext = ".cmnt"
  int flag_cm1, flag_cm2, flag_cm3  # 1:shutter trouble 2:bias jump 3:other

  int flag_inquiry    # 1: 'flag_overwrite' applys to all image
  int flag_overwrite  # 0:skip 1:overwrite
  int n_version       # IRAF version (210 or 211)

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------------------------" )
  print( " Subtracting dark image" )
  print( "---------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------------------------", >> logfn )
  print( " Subtracting dark image", >> logfn )
  print( "---------------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  darklist = envget( "_darklist" )
  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn = mktemp( "hbs.tmp.fn." )
  thres_hot = real( envget( "_hotpix" ) )

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    print( "Cannot find "//listoflist//"   Please do hbsgroup", >> logfn )
    bye
  }
  if( access( darklist ) == 0 )
  {
    print( "Cannot find "//darklist//"   Please do hbsdarkave" )
    print( "Cannot find "//darklist//"   Please do hbsdarkave", >> logfn )
    bye
  }

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "NX0" )
    {
      x0 = real( buf3 )
    }
    if( buf2 == "NY0" )
    {
      y0 = real( buf3 )
    }
  }
  if( x0*y0 == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    printf( "Read error in '%s'\n", fn_def, >> logfn )
    bye
  }

#----- Dark Region Definition

  x1 = 0.05 * x0
  x2 = 0.15 * x0
  y1 = 0.05 * y0
  y2 = 0.15 * y0
  x3 = 0.85 * x0
  x4 = 0.95 * x0
  y3 = 0.85 * y0
  y4 = 0.95 * y0

#  x1 = 0.01 * x0
#  x2 = 0.09 * x0
#  y1 = 0.01 * y0
#  y2 = 0.09 * y0
#  x3 = 0.91 * x0
#  x4 = 0.99 * x0
#  y3 = 0.91 * y0
#  y4 = 0.99 * y0


#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del1" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del1" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n", >> logfn )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n" )

    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- Search the appropriate dark image

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" )
    {

    #--- comment for each group

      fn_cmnt = hbslist[i]//cmntext
      cmnt = ""
      flag_cm1 = 0
      flag_cm2 = 0
      flag_cm3 = 0


      flag_hotpix = 0
      lastcnt = -9999.0

      if( flag_ver == 1 )
        printf( "Now searching appropriate dark... " )
      printf( "Now searching appropriate dark... ", >> logfn )
      flag_d = 0
      shortdiaph = substr( diaph[i]//"____", 1, 7 )

      if( flag_ver == 1 )
        printf( "Level-1 " )
      printf( "Level-1 ", >> logfn )
      list = darklist
      while( fscan( list, fn, d_title, d_exptime,\
             d_diaph, d_filter, d_slit ) != EOF )
      {
  #---level-1
        d_shortdiaph = substr( d_diaph//"____", 1, 7 )
        if( d_title == "------" )
	{
          if( exptime[i] == d_exptime )
	  {
            flag_d = 1
            if( flag_ver == 1 )
              printf( "detected\n" )
            printf( "detected\n", >> logfn ) 
            break
	  }
	}
        else
	{
          if( exptime[i] == d_exptime && title[i] == d_title &&\
	      diaph[i] == d_diaph )
          {
            flag_d = 1
            if( flag_ver == 1 )
              printf( "detected\n" )
            printf( "detected\n", >> logfn )
            break
          }
	}
      }

  #---level-2
      if( d_title != "------" )
      {
        if( flag_d == 0 )
        {
          if( flag_ver == 1 )
            printf( "Level-2 " ) 
          printf( "Level-2 ", >> logfn ) 
          list = darklist
          while( fscan( list, fn, d_title, d_exptime,\
                 d_diaph, d_filter, d_slit ) != EOF )
          {
            if( exptime[i] == d_exptime && title[i] == d_title &&\
	        shortdiaph == d_shortdiaph )
            {
              flag_d = 1
              if( flag_ver == 1 )
                printf( "detected\n" )
              printf( "detected\n", >> logfn )
              break
            }
          }
        }
      }

  #---level-3
      if( d_title != "------" )
      {
        if( flag_d == 0 )
        {
          if( flag_ver == 1 )
            printf( "Level-3 " ) 
          printf( "Level-3 ", >> logfn ) 
          list = darklist
          while( fscan( list, fn, d_title, d_exptime,\
                 d_diaph, d_filter, d_slit ) != EOF )
          {
            if( exptime[i] == d_exptime )
            {
              flag_d = 1
              if( flag_ver == 1 )
                printf( "detected\n" )
              printf( "detected\n", >> logfn )
              break
            }
          }
        }
      }


#----- In the case of no appropriate dark image

      if( flag_d == 0 )
      {
        print( "" )
        print( "Could not find appropriate dark image for ", hbslist[i] )
        print( "", >> logfn )
        print( "Could not find appropriate dark image for ", hbslist[i],\
               >> logfn )

input_darkimage :
        print( "---------------- Dark List-----------------\n" )
	type( darklist )
        print( "-------------------------------------------\n" )
        printf( "Select dark image ([n]:Skip this group) >> " )

        print( "---------------- Dark List-----------------\n", >> logfn )
	type( darklist, >> logfn )
        print( "-------------------------------------------\n", >> logfn )
        printf( "Select dark image ([n]:Skip this group) >> ", >> logfn )
        x = scan( fn )
        printf( "%s\n", fn, >> logfn )
        if( fn == "n" )
          next
        if( access( fn ) || access( fn//".imh") )
        {
          imgets( fn, 'DATA-TYP' )
          d_datatyp = imgets.value
          imgets( fn, 'HBSTITLE' )
          d_title = imgets.value
          imgets( fn, 'EXPTIME' )
          d_exptime = imgets.value
          imgets( fn, 'DIAPHRAG' )
          d_diaph = imgets.value
          imgets( fn, 'FILTER' )
          d_filter = imgets.value
          imgets( fn, 'SLIT-WID' )
          d_slit = imgets.value
          print( "  DATA-TYP=", d_datatyp," HBSTITLE=", d_title,\
                 " EXPTIME=", d_exptime, " DIAPHRAGM=", d_diaph,\
                 "FILTER=", d_filter, "SLIT=", d_slit )
          print( "  DATA-TYP=", d_datatyp," HBSTITLE=", d_title,\
                 " EXPTIME=", d_exptime, " DIAPHRAGM=", d_diaph,\
                 "FILTER=", d_filter, "SLIT=", d_slit, >> logfn )
          if( exptime[i] != d_exptime )
          {
            print( "Warning: header item EXPTIME aren't identical" )
            print( "Warning: header item EXPTIME aren't identical", >> logfn )
          }
          if( d_datatyp != "DARK" )
          {
            print( "Warning: header item DATA-TYP aren't 'DARK'" )
            print( "Warning: header item DATA-TYP aren't 'DARK'", >> logfn )
          }
          printf( "Continue ? (y/n)  >> " )
          printf( "Continue ? (y/n)  >> ", >> logfn )
          x = scan( buf )
          while( buf != "y" && buf != "n" )
            x = scan( buf )
          if( buf == "n" )
            goto input_darkimage
          dark_fn = fn
        }
        else
        {
          print( fn, " isn't found" )
          print( fn, " isn't found", >> logfn )
          goto input_darkimage
        }
      }

#----- In the case of found appropriate dark image

      else
         dark_fn = fn

#----- Each Group

      if( flag_ver == 1 )
      {
        print( "--- OBJECT LIST : "//hbslist[i]//" ---" )
        print( "     DARK IMAGE : "//dark_fn )

      }
      printf( "%s  %s\n", hbslist[i], dark_fn, >> darklog )
      print( "--- OBJECT LIST : "//hbslist[i]//" ---", >> logfn )
      print( "     DARK IMAGE : "//dark_fn, >> logfn )
      printf( "%s  %s\n", hbslist[i], dark_fn, >> logfn )
      sed( 's/fit.imh//', hbslist[i], >tmp_list )

#----- hot-pixel correction table

      if( access( tmp_hotpix ) == 1 )
        delete( tmp_hotpix, ver- )

      imstat( dark_fn, fields="max", format-, >> tmp_hotpix )
      list = tmp_hotpix
      if( fscan( list, x ) != 1 )
      {
        printf( "Read Error in %s\n", tmp_hotpix )
        printf( "Read Error in %s\n", tmp_hotpix, >> logfn )
	bye
      }
      delete( tmp_hotpix, ver- )

      if( x > thres_hot )
      {
        flag_hotpix = 1
        printf( "     HOT-PIXELs (>%5.0fADU) found. Making table... ",\
	        thres_hot )
        printf( "     HOT-PIXELs (>%5.0fADU) found. Making table... ",\
	        thres_hot, >> logfn )
	if( access( dark_fn ) )
	  delete( dark_fn, ver- )

        if( n_version == 210 )
	{
	  wfits( dark_fn, dark_fn, bscale=1.0, bzero=0.0, long_header-,
                 short_header-, bitpix=-32, scale-, autoscale- )
	}
	else
	{
          if( n_version == 211 )
	  {
	    wfits( dark_fn//".imh", dark_fn, bscale=1.0, bzero=0.0,\
	           long_header-, fextn="",\
                   short_header-, bitpix=-32, scale-, autoscale- )
	  }
	  else
	  {
            printf( "IRAF version erorr (%d)\n", n_version )
            printf( "IRAF version erorr (%d)\n", n_version, >> logfn )
            bye
	  }
	}
        hbshotpxtab( dark_fn, thres_hot, tmp_hotpix )
	delete( dark_fn, ver- )
      }

#----- Scanning Each List

      n_image = 0
      list = tmp_list
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      for( j=1; j<=n_image; j+=1 )
      {
        fn1 = fn_stack[j]//"fit"
        fn2 = fn_stack[j]//"d"

	if( j < n_image )
         fnnext = fn_stack[j+1]//"d"
	else
        fnnext = "none"

        if( access( fn1//".imh" ) == 0 ){
          if( flag_ver == 1 )
          {
            printf( "%s--(not exist. skip.)  ", fn1 )
            printf( "%s--(not exist. skip.)  ", fn1, >> logfn )
          }
          next
        }

        if( access( fn2//".imh" ) == 1 ){

          if( flag_overwrite == 0 )
          {
            if( fnnext != "none" && access( fnnext//".imh" ) == 0 )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
	               fn2 )
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
	               fn2, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn2 )
              printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn2 )
              printf( "\n%s already exist.\n", fn2, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              printf( "%s\n", buf )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn2 )
                printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn2 )
                printf( "%s--(already exists. skip.)  ", fn2, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn2 )
          printf( "%s(overwrite)  ", fn2, >> logfn )
          imdelete( fn2, ver- )
        }
        else
          if( flag_ver == 1 )
            printf( "%s  ", fn2 )
          printf( "%s  ", fn2, >> logfn )

#----- Subtracting dark image

        imarith( fn1, "-", dark_fn, fn2,
                 pixtype="real", calctype="real", ver- )

#----- Hot-Pixel Correction

##        if( flag_hotpix == 1 )
##          fixpix( fn2, tmp_hotpix )

#----- Bias Correction

        imstatistics( fn2//"["//x1//":"//x2//","//y1//":"//y2//"]",\
	        format-, fields="midpt", binwidth=0.1, >> tmp_fn )
        list = tmp_fn
        if( fscan( list, cnt1 ) != 1 )
        {
          printf( "Imstatistics-file read error for %s\n", fn2 )
          printf( "Imstatistics-file read error for %s\n", fn2, >> logfn )
          bye
        }
        delete( tmp_fn, ver- )

        imstatistics( fn2//"["//x1//":"//x2//","//y3//":"//y4//"]",\
	        format-, fields="midpt", binwidth=0.1, >> tmp_fn )
        list = tmp_fn
        if( fscan( list, cnt2 ) != 1 )
        {
          printf( "Imstatistics-file read error for %s\n", fn2 )
          printf( "Imstatistics-file read error for %s\n", fn2, >> logfn )
          bye
        }
        delete( tmp_fn, ver- )

        imstatistics( fn2//"["//x3//":"//x4//","//y1//":"//y2//"]",\
	        format-, fields="midpt", binwidth=0.1, >> tmp_fn )
        list = tmp_fn
        if( fscan( list, cnt3 ) != 1 )
        {
          printf( "Imstatistics-file read error for %s\n", fn2 )
          printf( "Imstatistics-file read error for %s\n", fn2, >> logfn )
          bye
        }
        delete( tmp_fn, ver- )

        imstatistics( fn2//"["//x3//":"//x4//","//y3//":"//y4//"]",\
	        format-, fields="midpt", binwidth=0.1, >> tmp_fn )
        list = tmp_fn
        if( fscan( list, cnt4 ) != 1 )
        {
          printf( "Imstatistics-file read error for %s\n", fn2 )
          printf( "Imstatistics-file read error for %s\n", fn2, >> logfn )
          bye
        }
        delete( tmp_fn, ver- )

        if( flag_ver == 1 )
          printf( "%7.3f %7.3f %7.3f %7.3f  ", cnt1, cnt2, cnt3, cnt4 )
        printf( "%7.3f %7.3f %7.3f %7.3f  ", cnt1, cnt2, cnt3, cnt4,\
                >> logfn )

        if( shorttyp[i] != "FLA" && shorttyp[i] != "COM" )
	{
	  if( abs( cnt2 - cnt1 ) > cntthresh || abs( cnt4 - cnt3 ) > cntthresh )
          {
	    printf( "## Abnormal!! (shutter trouble?) Deleted.\n" )
	    printf( "%s %7.3f %7.3f %7.3f %7.3f ## Abnormal!! (shutter trouble?)  Deleted.\n",
	    fn2, cnt1, cnt2, cnt3, cnt4, >> darklog )

	    printf( "%s %7.3f %7.3f %7.3f %7.3f ## Abnormal!! (shutter trouble?)  Deleted.\n",
	    fn2, cnt1, cnt2, cnt3, cnt4, >> logfn )

	    imdelete( fn2, ver- )
	    flag_cm1 = 1
	    next
	  }
	  if( abs( cnt3 - cnt1 ) > cntthresh || abs( cnt4 - cnt2 ) > cntthresh )
          {
	    if( flag_ver == 1 )
	      printf( "## less reliable  " )
            printf( "## less reliable  ", >> logfn )
            printf( "## less reliable  ", >> darklog )
	    flag_cm3 = 1
	  }
	}
        cnt = ( cnt1 + cnt2 + cnt3 + cnt4 ) / 4.0

        if( lastcnt > -1000 && abs( cnt - lastcnt ) > ( 10 * cntthresh ) )
        {
          if( shorttyp[i] != "FLA" && shorttyp[i] != "COM" )
          {
	    if( flag_ver == 1 )
	      printf( "## Abnormal bias jump. Deleted." )
            printf( "## Abnormal bias jump. Deleted. ", >> logfn )
            printf( "## Abnormal bias jump. Deleted.", >> darklog )
	    flag_cm2 = 2
            imdelete( fn2, ver- )
	    next
	  }
	  else
          {
	    if( flag_ver == 1 )
	      printf( "## Maybe stray light exist. Less reliable  " )
            printf( "## Maybe stray light exist. Less reliable  ", >> logfn )
            printf( "## Maybe stray light exist. Less reliable  ", >> darklog )
	    flag_cm2 = 1
	  }
	}
        else
	{
          if( lastcnt > -1000 && abs( cnt - lastcnt ) > ( 0.5 * cntthresh ) )
          {
	    if( flag_ver == 1 )
	      printf( "## less reliable (bias jump???)  " )
            printf( "## less reliable (bias jump???) ", >> logfn )
            printf( "## less reliable (bias jump???) ", >> darklog )
	    flag_cm2 = 1
	  }
	}

        if( flag_ver == 1 )
	  printf( "%7.3f  ", cnt )
        printf( "%7.3f  ", cnt, >> logfn )

        imarith( fn2, "-", cnt, fn2, pixtype="real",\
	         calctype="real", ver- )

	if( access( fn2//".imh" ) == 0 )
	{
	  printf( "Dark-subtraction Failed. Abnormal end.\n" )
	  printf( "Dark-subtraction Failed. Abnormal end.\n", >> logfn )
	  bye
	}

        printf( "%s  %7.3f %7.3f %7.3f %7.3f  %7.3f\n", fn2, cnt1, cnt2, cnt3, cnt4, cnt, >> darklog )

        hedit( fn2, "HISTORY", "Dark Subt.("//dark_fn//\
	       ") and Const Subt.("//cnt//")", add+, delete-, verify-,\
	       show-, update+ )

#----- Correction of defect pixels (since 2001/02)

        imstat( fn2, fields="min", format- ) | scan( cnt1 )
	if( cnt1 < dthresh )
	{
	  imcopy( fn2, tmp_fn, ver- )
	  imreplace( tmp_fn, 0.0, lower=dthresh, upper="INDEF" )
	  imreplace( tmp_fn, 1.0, lower="INDEF", upper=dthresh )
	  fixpix( fn2, masks=tmp_fn, linterp="INDEF",\
	          cinterp="INDEF", ver- )
	  imdelete( tmp_fn, ver- )
         if( flag_ver == 1 )
	    printf( "Pixel defect correcterd  " )
	  printf( "Pixel defect correcterd  ", >> logfn )
	}

	if( flag_erase == 1 )
	  imdelete( fn1, ver- )

        if( flag_ver == 1 )
          printf( "\n" )
        printf( "\n", >> logfn )

	lastcnt = cnt
     }
      if( flag_hotpix == 1 )
        delete( tmp_hotpix, ver- )
      if( flag_ver == 1 )
        printf( "\n" )
      printf( "\n", >> logfn )



 
    #---- comment file for each group
      if( flag_cm1 == 1 )
        cmnt = cmnt//"_Shutter_trouble"
      if( flag_cm2 == 1 )
        cmnt = cmnt//"_Bias_jump"
      if( flag_cm2 == 2 )
        cmnt = cmnt//"_Severe_Bias_jump"
      if( flag_cm3 == 1 )
        cmnt = cmnt//"_Other"

#      if( access( fn_cmnt ) == 1 )
#      {
#        list = fn_cmnt
#        if( fscan( list, cmnt ) != 1 ){}
#        delete( fn_cmnt, ver- )
#      }
      if( cmnt != "" )
        printf( "%s", cmnt, >> fn_cmnt )
    }
  }

end
