#
#	HBS REDUCTION SUB ROUTINE
#
#	File conversion for CL : hbsfconv.cl
#
#       1997/11/01      K.Kawabata    Ver.1.00  for CRYOCAM
#       1997/11/04      K.Kawabata    Ver.1.10  combined code
#       1998/04/20      K.Kawabata    Ver.2.00  unification
#       1998/05/31      K.Kawabata    Ver.2.10
#       2000/02/29      K.Kawabata    Ver.3.00
#       2005/04/07      M.Isogai      Ver 3.10  for iraf v2.12
#       2005/11/02      M.Isogai      Ver 4.00  include andorconv
#

procedure hbsfconv

begin
  
  int c, j
  file fn, fnnext
  file tmp_fn
  int flag_erase = 0
  string fn_stack[5000]
  int n_image
  string buf, buf1, buf2, buf3
  string caldir, fn_def
  int nccd = 0       # 1:TC215 2:SITe 3:Andor


  int flag_inquiry   # 1: 'flag_overwrite' applies to all image
#  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

  file logfn

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

#-- If NCCD=3, execute Andor Conversion  2005 11/01 -- 
#

  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )

#----- read difinition file
  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  nccd = 1        # for old 'hbsdefine.h'

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
   {
    if( buf2 == "NCCD" )
      {
      nccd = int( buf3 )
      }
   }
  if( nccd == 3 )
    {
      printf( "\n" )
      printf( "-----------------\n" )
      printf( " AndorCCD File Conversion\n" )
      printf( "-----------------\n" )
      printf( "\n" )

      printf( "\n", >> logfn )
      printf( "-----------------\n", >> logfn )
      printf( " File Conversion\n", >> logfn )
      printf( "-----------------\n", >> logfn )
      printf( "\n", >> logfn )

      andorconv
      }

#------------------------------------
  printf( "\n" )
  printf( "%s %s\n", buf1, buf2 )
  printf( "-----------------\n" )
  printf( " File Conversion\n" )
  printf( "-----------------\n" )
  printf( "\n" )

  printf( "\n", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  printf( "-----------------\n", >> logfn )
  printf( " File Conversion\n", >> logfn )
  printf( "-----------------\n", >> logfn )
  printf( "\n", >> logfn )

  tmp_fn = mktemp( "hbs.tmp.fn." )


#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n",\
              >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
	  >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del1" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del1" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del1) Retry.\n",\
      >> logfn )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n" )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'hbsred'\n", >> logfn )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- cryocam fts file --> astronomical FITS file


  printf( "Cryofits3 proceeding... ", >> logfn )
  cryofits3
  printf( "done.\n", >> logfn )

  if( flag_erase == 1 )
  {
    printf( "Now scanning and deleting old files\n" )
    printf( "Now scanning and deleting old files\n", >> logfn )
    sections( "*.fts", option="fullname", > tmp_fn )
    list = tmp_fn
    while( fscan( list, buf1 ) != EOF )
    {
      c = strlen( buf1 )
      buf2 = substr( buf1, 1, ( c - 3 ) )//"fit"
      if( access( buf2 ) == 1 )
        delete( buf1, ver- )
    }
    delete( tmp_fn, ver- )
  }

#----- rfits

  sections( "*.fit", option="fullname", > tmp_fn )
 
  list = tmp_fn
  n_image = 0
  while( fscan( list, buf1 ) != EOF )
  {
    n_image += 1
    fn_stack[n_image] = buf1
  }
  delete( tmp_fn, ver- )

  for( j=1; j<=n_image; j+=1 )
  {
    fn = fn_stack[j]

    if( j < n_image )
      fnnext = fn_stack[j+1]
    else
      fnnext = "none"


    if( access( fn//".imh" ) == 1 ){

      if( flag_overwrite == 0 )
      {
        if( fnnext != "none" && access( fnnext//".imh" ) == 0 )
        {
	  printf( "\n%s: Maybe insufficient proccess last time.\n", fn )
	  printf( "\n%s: Maybe insufficient proccess last time.\n", fn,\
	   >> logfn )
          goto inquiry
        }

        if( flag_inquiry == 1 )
        {
          if( flag_ver == 1 )
	  {
            printf( "%s--(already exists. skip.)  ", fn )
	  }
          printf( "%s--(already exists. skip.)  ", fn, >> logfn )
          next
        }
        else
	{
          printf( "\n%s already exist.\n", fn )
          printf( "\n%s already exist.\n", fn, >> logfn )
      inquiry:
          printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
          printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ",\
	  >> logfn )
          c = scan( buf1 )
          while( buf1 != "Y" && buf1 != "y" && buf1 != "n" && buf1 != "N" )
            c = scan( buf1 )
	  printf( "%s\n", buf1, >> logfn )
          if( buf1 == "Y" )
          {
            flag_inquiry = 1
            flag_overwrite = 1
          }
          if( buf1 == "N" )
          {
            flag_inquiry = 1
            flag_overwrite = 0
            if( flag_ver == 1 )
              printf( "%s--(already exists. skip.)  ", fn )
            printf( "%s--(already exists. skip.)  ", fn, >> logfn )
            next
          }
          if( buf1 == "n" )
          {
            if( flag_ver == 1 )
              printf( "%s--(already exists. skip.)  ", fn )
            printf( "%s--(already exists. skip.)  ", fn, >> logfn )
            next
          }
        }
      }
      if( flag_ver == 1 )
        printf( "%s(overwrite)  ", fn )
      printf( "%s(overwrite)  ", fn, >> logfn )
      imdelete( fn//".imh", ver- )
    }
    else
    {
#      if( flag_ver == 1 )
#        printf( "%s  ", fn )
      printf( "rfits %s  ", fn, >> logfn )
    }

    if( n_version == 210 )
    {
      rfits( fn, "", fn, make_image+, long_header-, short_header+,\
             datatype="real", blank=0., scale+, oldirafname-, offset=0 )
    }
    else
    {
      if( n_version == 211 )
      {
        rfits( fn, "", fn//".imh", make_image+, long_header-, short_header+,\
               datatype="real", blank=0., scale+, oldirafname-, offset=0 )
      }
      else
      {
        printf( "IRAF version error (%d)\n", n_version )
        printf( "IRAF version error (%d)\n", n_version, >> logfn )
	bye
      }
    }

    if( access( fn//".imh" ) == 0 )
    {
      printf( "Rfits failed. Abnormal end.\n" )
      printf( "Rfits failed. Abnormal end.\n", >> logfn )
      bye
    }

    if( flag_erase == 1 )
      delete( fn, ver- )
  }

end
