#
#       HBS REDUCTION SUB ROUTINE
#
#       Scan and extract two-dimensional spectra of FLAT-field frames
#                                               : hbsflatext.cl
#
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#

procedure hbsflatext

begin

  file fn, fn1
  int i
  file flatlist1, flatlist2
  int n_list
  string hbsfile[50]
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8, buf9
  string caldir, rdir
  file fn_def
  int y1, y2, y3, y4
  int y1_2HL, y2_2HL, y3_2HL, y4_2HL
  int y1_2S2, y2_2S2, y3_2S2, y4_2S2
  int ydim

  file fni1, fni2, fni3, fni4

#--- initial settings

  print( "" )
  print( "-------------------------------------------" )
  print( " scan and extract apertures of FLAT images" )
  print( "-------------------------------------------" )
  print( "" )

  buf = ""
  flatlist1 = "hbs.tmp.flat1"
  flatlist2 = "hbs.tmp.flat2"
  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )


#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""
  ydim = 0
  y1_2HL = 0
  y3_2HL = 0
  y1_2S2 = 0
  y3_2S2 = 0

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "NY2" )
    {
      ydim = int( buf3 )
    }
    if( buf2 == "DY1_2HL" )
    {
      y1_2HL = int( buf3 )
    }
    if( buf2 == "DY2_2HL" )
    {
      y3_2HL = int( buf3 )
    }
    if( buf2 == "DY1_2S2" )
    {
      y1_2S2 = int( buf3 )
    }
    if( buf2 == "DY2_2S2" )
    {
      y3_2S2 = int( buf3 )
    }
  }
  if( (ydim*y1_2HL*y3_2HL*y1_2S2*y3_2S2) == 0 )
  {
    printf( "Read error in '%s' for 'NY2' or 'DY1' or 'DY2'\n", fn_def )
    bye
  }
  y2_2HL = y1_2HL + ydim - 1
  y4_2HL = y3_2HL + ydim - 1
  y2_2S2 = y1_2S2 + ydim - 1
  y4_2S2 = y3_2S2 + ydim - 1

#----- Scanning the FLAT list file

  if( access( flatlist1 ) == 1 )
    delete( flatlist1, ver- )
  if( access( flatlist2 ) == 1 )
    delete( flatlist2, ver- )

  sections( "FLAT.*.px.imh", option="fullname", >> flatlist1 )
  sed( "s/px\.imh//", flatlist1, >> flatlist2 )

  delete( flatlist1, ver- )

  n_list = 0
  list = flatlist2
  while( fscan(list, buf1) != EOF )
  {
    n_list += 1
    hbsfile[n_list] = buf1
  }

  delete( flatlist2, ver- )

  printf( "refereing calib's in %s\n", caldir )

#----- main routine

  flag_first = 1

  for( i=1; i<=n_list; i+=1 )
  {
    y1 = 0
    y2 = 0
    y3 = 0
    y4 = 0

    rdir = caldir

    fn = hbsfile[i]

    fn1 = fn//"px"
    fni1 = fn//"px.1"
    fni2 = fn//"px.2"
    fni3 = fn//"px.3"
    fni4 = fn//"px.4"

    printf( "%s  ", fn )

    imgets( fn1, "DIAPHRAG" )
    buf = imgets.value

    if( substr( buf, 1, 6 ) == "2HOLES" )
    {
      y1 = y1_2HL
      y2 = y2_2HL
      y3 = y3_2HL
      y4 = y4_2HL
    }
    if( substr( buf, 1, 7 ) == "2SLIT02" )
    {
      y1 = y1_2S2
      y2 = y2_2S2
      y3 = y3_2S2
      y4 = y4_2S2
    }

    if( y1*y2*y3*y4 == 0 )
    {
      printf( "Unknown DIAPHRAG type (%s). Skip.\n", buf )
      next
    }

    if( access( fni1//".imh" ) == 1 )
      imdelete( fni1, ver- )
    if( access( fni2//".imh" ) == 1 )
      imdelete( fni2, ver- )
    if( access( fni3//".imh" ) == 1 )
      imdelete( fni3, ver- )
    if( access( fni4//".imh" ) == 1 )
      imdelete( fni4, ver- )

    hbsapext( fn1, rdir, fni1, fni2, fni3, fni4, y1, y2, y3, y4 )

    hedit( fni1, "HISTORY", "Spectrum Extracted: reference in "\
           //rdir, delete-, verify-, show-, update+ )
    hedit( fni2, "HISTORY", "Spectrum Extracted: reference in "\
           //rdir, delete-, verify-, show-, update+ )
    hedit( fni3, "HISTORY", "Spectrum Extracted: reference in "\
           //rdir, delete-, verify-, show-, update+ )
    hedit( fni4, "HISTORY", "Spectrum extracted: reference in "\
           //rdir, delete-, verify-, show-, update+ )

  }
 
end
