#
#       HBS REDUCTION SUB ROUTINE
#
#       Re-identify for wavelength calibration : hbspostidentify.cl
#
#       1997/11/19      K.Kawabata  Ver.0.10
#

procedure hbsostidentify

begin

  file listoflist = "HBS.list"
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  int c, i, j, n, n_list, n_image
  int x_order = 2
  int f_wid = 5
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  file fns
  file tmp_list, tmp_lista
  file ref_fn = "dispref.od.0"
  string buf=""
  string fitsec="middle line"
  string wavelist
  string ref_dir
  int ydim
  file fn_def = "home$prog/hbsdefine.h"
  real sw1, ew1, dw1

  tmp_lista = mktemp( "hbs.tmp.list." )
  tmp_list = mktemp( "hbs.tmp.list." )

  print( "" )
  print( "------------------------------------------------------" )
  print( " identifucation for one-dimensional comparison images" )
  print( "------------------------------------------------------" )
  print( "" )


#----- check wavelength data file

  sw1 = 0.0
  ew1 = 0.0
  dw1 = 0.0

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find %s\n", ref_def )
    bye
  }
  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
      sw1 = real( buf3 )
    if( buf2 == "EW" )
      ew1 = real( buf3 )
    if( buf2 == "DW" )
      dw1 = real( buf3 )
  }
  if( sw1 * dw1 * dw1 <= 0.00001 )
  {
    printf( "read error in %s\n", ref_def )
    bye
  }
  else
    printf( "sw = %6.1f  ew = %6.1f  dw = %6.3f\n", sw1, ew1, dw1 )


#----- delete old images

findoldimage:

  if( access( ref_fn//".imh" ) )
  {
    printf( "old reference images '%s' exist. Use this? (y/n) >> ", ref_fn )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
    {
      imdelete( ref_fn, ver- )
      goto findoldimage
    }
  }
  else
  {

#----- Scanning the list file

    if( access( listoflist ) == 0 )
    {
      print( "Cannot find "//listoflist//"   Please do hbsgroup" )
      bye
    }
    n_list = 0
    list = listoflist
    while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
    {
      if( substr( buf1, 1, 3 ) != "HBS" )
      {
        print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
        break
      }
      if( buf2 != "OBJ" && buf2 != "COM" )
        next
      n_list += 1
      hbslist[n_list] = buf1
      shorttyp[n_list] = buf2
      title[n_list] = buf3
      exptime[n_list] = buf4
      diaph[n_list] = buf5
      filter[n_list] = buf6
      slit[n_list] = buf7
    }

#---- selection of images for summation

    if( access( tmp_lista ) == 1 )
      delete( tmp_lista, ver- )

    for( i=1; i<=n_list; i+=1 )
    {
     printf( "%2d: %16s  %6s  %s\n", i, hbslist[i], title[i], diaph[i] )
    }
nextstep1:
    printf( "Select a group to be sum ([e]:end selection) >> " )
    while( scan( buf ) != 1 ){}
    if( buf == "e" )
      goto nextstep2
    k = int( buf )
    if( k < 1 || k > n_list )
      goto nextstep1
    printf( "object %s   diaphragm %s  slit width %s\n",\
            title[k], diaph[k], slit[k] )
    if( real( slit[k] ) > 1.0 )
    {
      printf( "Warning: Slit width of %s is too wide.\n", hbslist[k] )
    }
    type( hbslist[k], >> tmp_lista )
    goto nextstep1

nextstep2:
  
#----- summing images

    sed( 's/\.fit\./\.px\.1\./', tmp_lista, > tmp_list )
    sed( 's/\.fit\./\.px\.2\./', tmp_lista, >> tmp_list )
    sed( 's/\.fit\./\.px\.3\./', tmp_lista, >> tmp_list )
    sed( 's/\.fit\./\.px\.4\./', tmp_lista, >> tmp_list )
    delete( tmp_lista, ver- )

    n_image = 0
    list = tmp_list
    while( fscan( list, buf) != EOF )
    {
      n_image += 1
    }
    printf( "Now making summed images  %d image(s)\n", n_image )
    imsum( input="@"//tmp_list, output=ref_fn, hparam="EXPTIME",\
           option="sum", ver- )
    imgets( ref_fn, 'i_naxis2' )
    ydim = int( imgets.value )
    blkavg( ref_fn, ref_fn, 1, ydim, option="sum" )
    delete( tmp_list, ver- )
  }

#----- select type

  print( "image type  [h]:Hg-Ne [i]:nebula [n]:TI-CCD >>" )
  c = scan( buf )
  while( buf != "h" && buf != "i" && buf != "n" )
    c = scan( buf )
  if( buf == "h" )
    wavelist = "home$etc/Hg-Ne.dat"
  if( buf == "i" )
    wavelist = "home$etc/hbsnebula.dat"
  if( buf == "n" )
    wavelist = "home$etc/hbscryocam1.dat"
  type( wavelist )

#----- identify the middle line of the first spectrum

  if( access( "database/fc"//ref_fn ) )
    delete( "database/fc"//ref_fn, ver- )
  if( access( "database/id"//ref_fn ) )
    delete( "database/id"//ref_fn, ver- )

  printf( "push [m] key on the feature, and input wavelength\n" )
  identify( ref_fn//".imh", section=fitsec, databas='database',\   
            coordli=wavelist, nsum=1, match=20.,\
            maxfeat=30, fwidth=f_wid, cradius=10., functio='chebyshev',\
            order=x_order, niterate=10, low_rej=3., high_rej=3., autowrite+,\
            grow=0., ftype='emission', mode='ql', thresho=3. )


  hedit( ref_fn, "REFSPEC1", ref_fn, delete-, add+,\
         verify-, show-, update+ )
  dispcor( ref_fn, ref_fn, linearize+, database="database",\
           table="", w1=sw1, w2=ew1, dw=dw1, nw=INDEF, log-,
           flux+, samedisp-, global-, ignoreaps+, confirm-, listonly-,\
           logfile="", ver- )

end
