#######################################################
#               HBS REDUCTION on IRAF                 #
#                                                     #
#              main program: hbsred.cl                #
#                                                     #
#            1998/04/20  Ver.2.00   K.Kawabata        #
#            1998/05/31  Ver.2.10   K.Kawabata        #
#            2000/02/29  Ver.3.00   K.Kawabata        #
#######################################################

procedure hbsred

begin

#----- const variables definision

  file histfile = "HBS.history"
  string procname[19]
  string initialproc = "hbsinit"
  int n_proc

#----- volatile variables definition

  int procid
  int procid0
  int procid1
  int c, last_c, i
  string buf = ""
  string buf1 = ""
  string buf2 = ""
  string buf3 = ""
  int flag_hist = 0

#----- constant variable definitions

  procname[1] = "hbsfconv"
  procname[2] = "hbsgroup"
  procname[3] = "hbsdarkave"
  procname[4] = "hbsdarksub"
  procname[5] = "hbstrim"
  procname[6] = "hbsbadpix"
  procname[7] = "hbspixsens"
  procname[8] = "hbsextract"
  procname[9] = "hbsskysub"
  procname[10] = "hbsarrange"
  procname[11] = "hbsflat2"
  procname[12] = "hbs1d"
  procname[13] = "hbscntcor"
  procname[14] = "hbsbin"
  procname[15] = "hbsiqugroup"
  procname[16] = "hbsiqumake"
  procname[17] = "hbsiquave"
  procname[18] = "hbsmktable"
  n_proc = 18

  procid = 1
  procid0 = 1
  procid1 = n_proc

#----- DELETE OLD TEMPORALY FILES

  printf( "Now clearing tempolary files...  Just a minute.\n" )
  imdelete('*tmp*.imh')
  delete('*tmp*')

#----- TITLE

  printf( "\n" )
  printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
  printf( "{     HBS REDUCTION KIT Ver 3.00 (Feb-29-2000)   }\n" )
  printf( "{                                                }\n" )
  printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"  )
  printf( "\n" )

#----- CHECK INITIALIZATION SCRIPT

  if( envget( "_initproc" ) != "DONE" )
  {
    printf( "Error: Not Done Initialization.\n" )
    printf( "Please do '%s'\n", initproc )
    bye
  }


#---- Read history file

  if( access( histfile ) == 1 )
  {
    printf( "\nHistory file detected.\n" )
    list = histfile
    c = fscan( list, buf1, procid0, buf2, buf3 )
    last_c = c
    while( c != EOF )
    {
      last_c = c
      c = fscan( list, buf1, procid0, buf2, buf3 )
    }
    if( last_c < 4 )
    {
      buf3 = "ABNORMAL_TERMINATION"
      printf( "%s\n", buf3, >> histfile )
    }

    if( procid0 < 1 || procid0 > n_proc )
    {
      printf( "Error: ProcID is wrong in %s. Ignored.\n", histfile )
    }
    else
    {
      flag_hist = 1
      printf( "LAST PROCEDURE NAME : %s (procID %d)\n",\
              procname[procid0], procid0 )
      printf( "               DATE : %s\n", buf1 )
      if( buf3 == "NORMAL_END" )
      {
        printf( "             STATUS : NORMAL END\n" )
        procid = procid0 + 1
      }
      else
      {
        printf( "             STATUS : ABNORMAL TERMINATION\n" )
        procid = procid0
      }
    }
  }

#----- Select Procedure

  if( flag_hist == 1 )
    printf( "[RET]:Continue.(from '%s') [1]:Customize >> ",\
            procname[procid] )
  else
    printf( "[RET]:New Reduction [1]:Customize >> " )
  buf = ""
  c = scan( buf )
  while( buf != "" && buf != "1" )
  {
    buf = ""
    c = scan( buf )
  }
  if( buf == "1" )
  {
    for( i=1; i<=n_proc; i+=1 )
      printf( "%2d  %s\n", i, procname[i] )
    printf( "Biginning Proc-ID number >> " )
    c = scan( procid )
    while( procid < 0 || procid > n_proc )
      c = scan( procid )
    printf( "End Proc-ID number >> " )
    c = scan( procid1 )
    while( procid1 < 0 || procid1 > n_proc )
      c = scan( procid1 )
  }

#----- main routine

  while( procid > 0 && procid <= procid1 )
  {
    if( access( "hbs.tmp.time" ) == 1 )
      delete( "hbs.tmp.time", ver- )
    time >> hbs.tmp.time

    list = "hbs.tmp.time"
    if( fscan( list, buf, buf1, buf2 ) != 3 )
    {
      printf( "Error: time task/read\n" )
      bye
    }
    delete( "hbs.tmp.time", ver- )
    printf( "\n%s-%s  %2d %s\n", buf2, buf1, procid, procname[procid] )
    printf( "%s-%s  %2d %s  ", buf2, buf1, procid, procname[procid],\
            >> histfile )
    
    switch( procid )
    {
      case 1:
        hbsfconv
        break
      case 2:
        hbsgroup
        break
      case 3:
        hbsdarkave
        break
      case 4:
        hbsdarksub
        break
      case 5:
        hbstrim
        break
      case 6:
        hbsbadpix
        break
      case 7:
        hbspixsens
        break
      case 8:
        hbsextract
        break
      case 9:
        hbsskysub
        break
      case 10:
        hbsarrange
        break
      case 11:
        hbsflat2
        break
      case 12:
        hbs1d
        break
      case 13:
        hbscntcor
        break
      case 14:
        hbsbin
        break
      case 15:
        hbsiqugroup
        break
      case 16:
        hbsiqumake
        break
      case 17:
        hbsiquave
        break
      case 18:
        hbsmktab
        break
      default:
        printf( "Procedure ID error !!!  Restart,please.\n" )
        printf( "(If you want re-examine, select manual mode)\n" )
	bye
    }
    printf( "NORMAL_END\n", >> histfile )
    procid += 1

  }

#----- ending

  if( procid > 18 )
  {
    printf( "Minimize disk space? (y/n) >> " )
    buf = ""
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      hbsclean

#   imdelete('*tmp*.imh')
#    delete('*tmp*')

    printf( "\n" )
    printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
    printf( "{   HBSRED FINISHED.   Let's Enjoy Astronomy.    }\n" )
    printf( "{                                                }\n" )
    printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
  }
  else
  {
    if( procid > 13 )
    {
      printf( "\n" )
      printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
      printf( "{   HBSRED terminated.    Just Over There!       }\n" )
      printf( "{                                                }\n" )
      printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
    }
    else
    {
      printf( "\n" )
      printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
      printf( "{   HBSRED terminated.   Long Way to Finish...   }\n" )
      printf( "{                                                }\n" )
      printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
    }
  }
  bye

end

