#
#       HBS REDUCTION SUB ROUTINE
#
#       make unpolarized sky image from images of hwp=0.0 and hwp=45
#                             : hbsskyunpol.cl
#
#       1996/12/30      Ver. 1.00    K.Kawabata
#       1996/01/01      Ver. 1.10    K.Kawabata  sequencial interp
#
#   !! the frame of 0 degree and one of 45 must be tabulated mutually !!
#   !! the frames must be ordered by time sequence !!
#   !! you can use 'dummy.imh" in list file if you don't have mutually !!
#
procedure hbsskyunpol( inlist, outframe )

file inlist	{ prompt = "Image list" }
file outframe	{ prompt = "head name of output unpolarized image" }

begin

  string fn[100]
  real jd[100]
  string hbslist
  string beta1, beta2
  file tmp_fn1, tmp_fn2
  string fnave
  real jdave
  int n, poshwp
  int last_poshwp = -999
  string outimage00, outimage45, outimage
  string extent
  real jd_tmp

  string buf, buf1, buf2, buf3, buf4, buf5
  int i
  int flag_log = 1  # 0:linear 1:log
  int flag_045 = 1  # 0:erase 0,45deg image 1:reserve

#----- scan list files and make averaged image

  fnave = mktemp( "hbs.tmp.image." )
  beta1 = mktemp( "hbs.tmp.image." )
  beta2 = mktemp( "hbs.tmp.image." )

  hbslist = inlist
  outimage = outframe

  outimage00 = outimage//"00"
  outimage45 = outimage//"45"


  print( "Now scanning "//hbslist )
  n = 0
  jdave = 0.0
  list = hbslist
  while( fscan( list, buf ) != EOF )
  {
    n += 1
    fn[n] = buf
#    imgets( fn[n], "JD-OBS" )
    imgets( fn[n], "JD" )
    jd_tmp = real( imgets.value )
    imgets( fn[n], "POS-HWP" )
    poshwp = int( imgets.value )
    jd[n] = jd_tmp - int( int( jd_tmp/10.0 )*10.0 )
    printf( "%2d %20s pa=%3d jd=%12.4f  day=%6.4f\n", n, fn[n], poshwp,\
    jd_tmp, jd[n] )
    if( poshwp == last_poshwp )
    {
      print( "0deg and 45deg must be ordered mutually" )
      bye
    }
    jdave += jd[n]
    last_poshwp = poshwp
  }

  if( flag_log == 0 )
    print( "linear mode" )
  else
    print( "log10 mode" )

  for( i=2; i<=(n-1); i+=2 )
  {

#----- 0---45---0 ( interpolate 0 )

 interp_0:

    if( substr( fn[i-1], 1, 5 ) == "dummy" )
      goto interp_45
    if( substr( fn[i], 1, 5 ) == "dummy" )
      goto interp_45
    if( substr( fn[i+1], 1, 5 ) == "dummy" )
      goto interp_45

    printf( "%d--%d--%d  %d=%s\n", (i-1), i, (i+1), i, fn[i] )

  #----- make beta2-image  ( y = beta1 + beta2 * x )

    if( flag_log == 0 )
    {
      tmp_fn1 = fn[i-1]
      tmp_fn2 = fn[i+1]
    }
    else
    {
      tmp_fn1 = mktemp( "hbs.tmp.image." )
      tmp_fn2 = mktemp( "hbs.tmp.image." )
      imfunc( fn[i-1], tmp_fn1, function="log10", ver- )
      imfunc( fn[i+1], tmp_fn2, function="log10", ver- )
    }
    imarith( tmp_fn1, "+", tmp_fn2, fnave, pixtype="real", calctype="real",\
             ver- )
    imarith( fnave, "/", 2, fnave, pixtype="real", calctype="real",\
             ver- )
    jdave = ( jd[i-1] + jd[i+1] ) / 2.0

    buf1 = mktemp( "hbs.tmp.image." )
    buf2 = mktemp( "hbs.tmp.image." )
    imarith( tmp_fn1, "-", fnave, buf1, pixtype="real", calctype="real",\
             ver- )
    imarith( buf1, "*", (jd[i-1]-jdave), buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( tmp_fn2, "-", fnave, buf2, pixtype="real", calctype="real",\
             ver- )
    imarith( buf2, "*", (jd[i+1]-jdave), buf2, pixtype="real",\
             calctype="real", ver- )

    imarith( buf1, "+", buf2, beta2, pixtype="real", calctype="real",\
             ver- )

    jd_tmp = (jd[i-1]-jdave)*(jd[i-1]-jdave) + (jd[i+1]-jdave)*(jd[i+1]-jdave)
    imarith( beta2, "/", jd_tmp, beta2, pixtype="real", calctype="real",\
             ver- )

    imdelete( buf1, ver- )
    imdelete( buf2, ver- )

    if( flag_log == 1 )
    {
      imdelete( tmp_fn1, ver- )
      imdelete( tmp_fn2, ver- )
    }


  #----- make beta1-image  ( y = beta1 + beta2 * x )

    buf1 = mktemp( "hbs.tmp.image." )
    imarith( beta2, "*", jdave, buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( fnave, "-", buf1, beta1, pixtype="real",\
             calctype="real", ver- )
    imdelete( buf1, ver- )


  #----- make interpolated images
  
    buf1 = mktemp( "hbs.tmp.image." )
    buf2 = "000"//i
    buf3 = outimage00//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )
    buf4 = outimage//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )
    buf5 = outimage45//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )

    imarith( beta2, "*", jd[i], buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( beta1, "+", buf1, buf3, pixtype="real",\
           calctype="real", ver- )
    imdelete( buf1, ver- )

    if( flag_log == 1 )
      imfunc( buf3, buf3, function="alog10", ver- )

    imarith( fn[i], "+", buf3, buf4, pixtype="real",\
           calctype="real", ver- )

    if( flag_045 == 1 )
      imcopy( fn[i], buf5, ver- )
    else
      imdelete( buf3, ver- )

    imdelete( beta1, ver- )
    imdelete( beta2, ver- )
    imdelete( fnave, ver- )


#----- 45---0---45 ( interpolate 45 )

 interp_45:

    if( (i+2) > n )
      goto interp_next

    if( substr( fn[i], 1, 5 ) == "dummy" )
      goto interp_next
    if( substr( fn[i+1], 1, 5 ) == "dummy" )
      goto interp_next
    if( substr( fn[i+2], 1, 5 ) == "dummy" )
      goto interp_next

    printf( "%d--%d--%d  %d=%s\n", i, (i+1), (i+2), (i+1), fn[i+1] )
     
  #----- make beta2-image  ( y = beta1 + beta2 * x )

    if( flag_log == 0 )
    {
      tmp_fn1 = fn[i]
      tmp_fn2 = fn[i+2]
    }
    else
    {
      tmp_fn1 = mktemp( "hbs.tmp.image." )
      tmp_fn2 = mktemp( "hbs.tmp.image." )
      imfunc( fn[i], tmp_fn1, function="log10", ver- )
      imfunc( fn[i+2], tmp_fn2, function="log10", ver- )
    }
    imarith( tmp_fn1, "+", tmp_fn2, fnave, pixtype="real", calctype="real",\
             ver- )
    imarith( fnave, "/", 2, fnave, pixtype="real", calctype="real",\
             ver- )
    jdave = ( jd[i] + jd[i+2] ) / 2.0


    buf1 = mktemp( "hbs.tmp.image." )
    buf2 = mktemp( "hbs.tmp.image." )
    imarith( tmp_fn1, "-", fnave, buf1, pixtype="real", calctype="real",\
             ver- )
    imarith( buf1, "*", (jd[i]-jdave), buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( tmp_fn2, "-", fnave, buf2, pixtype="real", calctype="real",\
             ver- )
    imarith( buf2, "*", (jd[i+2]-jdave), buf2, pixtype="real",\
             calctype="real", ver- )

    imarith( buf1, "+", buf2, beta2, pixtype="real", calctype="real",\
             ver- )

    jd_tmp = (jd[i]-jdave)*(jd[i]-jdave) + (jd[i+2]-jdave)*(jd[i+2]-jdave)
    imarith( beta2, "/", jd_tmp, beta2, pixtype="real", calctype="real",\
             ver- )

    imdelete( buf1, ver- )
    imdelete( buf2, ver- )

    if( flag_log == 1 )
    {
      imdelete( tmp_fn1, ver- )
      imdelete( tmp_fn2, ver- )
    }


  #----- make beta1-image  ( y = beta1 + beta2 * x )

    buf1 = mktemp( "hbs.tmp.image." )
    imarith( beta2, "*", jdave, buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( fnave, "-", buf1, beta1, pixtype="real",\
             calctype="real", ver- )
    imdelete( buf1, ver- )


  #----- make interpolated images
  
    buf1 = mktemp( "hbs.tmp.image." )
    buf2 = "000"//(i+1)
    buf3 = outimage45//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )
    buf4 = outimage//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )
    buf5 = outimage00//"."//substr( buf2, ( strlen( buf2 ) - 2 ),\
           strlen( buf2 ) )

    imarith( beta2, "*", jd[i+1], buf1, pixtype="real",\
             calctype="real", ver- )
    imarith( beta1, "+", buf1, buf3, pixtype="real",\
           calctype="real", ver- )
    imdelete( buf1, ver- )

    if( flag_log == 1 )
      imfunc( buf3, buf3, function="alog10", ver- )
      
    imarith( fn[i+1], "+", buf3, buf4, pixtype="real",\
           calctype="real", ver- )

    if( flag_045 == 1 )
      imcopy( fn[i+1], buf5, ver- )
    else
      imdelete( buf3, ver- )

    imdelete( beta1, ver- )
    imdelete( beta2, ver- )
    imdelete( fnave, ver- )

  interp_next:

  }


end
