/************************************************************
    Appending the header into FITS-FILE produced by CRYOCAM

        Ver 1.0  based on cryocam3.c        2005.09.08  M.Isogai
	Ver 1.1  convert signed to unsigned 2005.09.09  M.Isogai
*************************************************************/

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<ctype.h>

#include"hbsdefine.h"

#ifndef longitude
#define longitude +2.375742      /* LONGITUDE OF OBSERVATORY */
#endif
#ifndef lattitude
#define lattitude +0.62831853    /* LATTITUDE OF OBSERVATORY */
#endif
#ifndef OBSERVATORY
#define OBSERVATORY "DODAIRA_OBSERVATORY"
#endif
#ifndef TELESCOPE
#define TELESCOPE "36INCH_REFLECTOR"
#endif

char str1[25], str2[25], str3[25], str4[25];

char *mid( char *, int, int );
char *convstr(char *);
char *fitsdate(const struct tm *);
char *fitstime(const struct tm *);
char *fitsdatemjd(const double);
char *fitstimemjd(const double);
char *fitsra(const double);
char  *fitsdec(const double);
double timetomjd(int, int, int, int, int, int);
double getmjdtm(struct tm *);
void mjdtotime(double, int *, int *, int *, int *, int *, int *);
double getst(double, double);
double getha(double, double);
double getzd(double, double, double);
double corrra(double, double, double, double);
double corrdec(double, double, double, double);
double getfz(double);
char *chartolower( char * );
void errlist( char *, char * );

char *mid(char *string, int n1, int n2 ){
  int i;
  for( i=0; i<=(n2-1); i++ )
    str1[i] = string[n1+i-1];
  str1[n2] = '\0';
  return( str1 );
}

/****** CONVERSION OF STRING FOR IRAF-HEADER (ADD QUOTATIONS) ******/
char *convstr(char *string){
  if( strlen( string ) < 8 ){
    sprintf( str1, "'%-8s'", string );
  }
  else{
    sprintf( str1, "'%s'", string );
  }
  return( str1 );
}

/****** CONVERSION from CONSTRUCT tm TYPE VARIABLE to FITS STRING ******/
char *fitsdate(const struct tm *tmptime){
  int tmpyear;
  /**** before 1999 ****
     sprintf(str2, "%02d/%02d/%02d", tmptime->tm_mday, (1 + tmptime->tm_mon), tmptime->tm_year);
  */
  /**** after 2000 ****/
  if( tmptime->tm_year > 70 )
    tmpyear = 1900 + tmptime->tm_year;
  else
    tmpyear = 2000 + tmptime->tm_year;
  sprintf(str2, "%04d-%02d-%02d", tmpyear, (1 + tmptime->tm_mon), tmptime->tm_mday );
  /**/
  return( str2 );
}
char *fitstime(const struct tm *tmptime){
  sprintf(str3, "%02d:%02d:%02d", tmptime->tm_hour, tmptime->tm_min, tmptime->tm_sec);
  return(str3);
}

/****** CONVERSION from MJD(double precision) to FITS STRING ******/
char *fitsdatemjd(const double mjd){
  int year, month, day, hour, min, sec;

  mjdtotime(mjd, &year, &month, &day, &hour, &min, &sec);

  /*
  if(year < 2000){
    year -= 1900;
    sprintf(str2, "%02d/%02d/%02d", day, month, year);
  }
  else{
    sprintf(str2, "%4d-%02d-%02d", year, month, day);
  }
  */
  sprintf(str2, "%04d-%02d-%02d", year, month, day);

  return(str2);
}

char *fitstimemjd(const double mjd){
  int year, month, day, hour, min, sec;

  mjdtotime(mjd, &year, &month, &day, &hour, &min, &sec);
  sprintf(str3, "%02d:%02d:%02d", hour, min, sec);
  return(str3);
}

/****** RA (rad) -> FITS STRING ******/
char *fitsra(const double a){
  double b;
  int h, m, s;

  b = a*57.2957795130824/15.0;
  h = b + 0.000001;	/* avoid numerical-round (?) */
  b = (b - h) * 60.0;
  m = b + 0.000001;
  b = (b - m) * 60.0;
  s = b + 0.000001;
  sprintf(str2, "%02d:%02d:%02d", h, m, s);
  return(str2);
}

/****** DEC (RAD) -> FITS STRING ******/
char  *fitsdec(const double a){
  double b;
  int d, m ,s;
  char sn='+';

  b = a;
  if(b < 0.0){
    sn='-';
    b = fabs(b);
  }
  b = b*57.2957795130824;
  d = b + 0.000001;	/* avoid numerical-round (?) */
  b = (b - d) * 60.0;
  m = b + 0.000001;
  b = (b - m) * 60.0;
  s = b + 0.000001;
  sprintf(str3, "%c%02d:%02d:%02d", sn, d, m, s);
  return(str3);
}

/******* CONVERSION from DATE, TIME to MJD(= JD - 2400000.5) ******/
double timetomjd(int year, int month, int day, int hour, int min, int sec){
  double mjd;
  long i1, i2, i3, i4;

  if(month <= 2) {year--; month+=12;}
  i1 = 365.25*year;
  i2 = year/400.0;
  i3 = year/100.0;
  i4 = 30.59*(month-2.0);
  mjd = i1 + i2 - i3 + i4 + day - 678912;
  mjd += (double)hour/24.0 + (double)min/1440.0 + (double)sec/86400.0;

  return(mjd);
}

/****** tm$B9=B$BN$+$iD>@\(BMJD$B$rF@$k$H$-$N%P%$%Q%94X?t(B ******/
double getmjdtm(struct tm *tmptime){
  double mjd;
  if(tmptime->tm_year > 70)
    mjd = timetomjd((1900 + tmptime->tm_year), (1 +tmptime->tm_mon), tmptime->tm_mday, tmptime->tm_hour, tmptime->tm_min, tmptime->tm_sec);
  else
    mjd = timetomjd((2000 + tmptime->tm_year), (1 +tmptime->tm_mon), tmptime->tm_mday, tmptime->tm_hour, tmptime->tm_min, tmptime->tm_sec);
  return(mjd);
}

/****** CONVERSION from MJD(= JD - 2400000.5)to DATE, TIME ******/
void mjdtotime(double mjd, int *year, int *month, int *day, int *hour, int *min, int *sec){

  int i, n;
  int ymonth[]={0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
  double submjd, subday;
  double sp1, sp2;
  double a1=0.0027379093;

  *year = a1 * mjd + 1858.877;
  *month = 1;
  *day = 0;
  submjd = timetomjd(*year, *month, *day, 0, 0, 0);
  sp1 = mjd - submjd;
  if((*year % 4) == 0)	ymonth[2] = 29;
  if((*year % 100) == 0 && (*year % 400) != 0) ymonth[2]=28;
  sp2 = 0;
  for(i=1; i<=12; i++){
    n = sp1;
    if((n - sp2 - ymonth[i]) <= 0) break;
    sp2 += ymonth[i];
  }
  *month = i;
  subday = sp1 - sp2;
  if(*month >= 13)	{*year += 1; *month -= 12;}
  if(subday < 1)	{*year -= 1; *month = 12; subday += 31;}
  *day = subday;
  subday -= *day;
  subday *= 24.0;
  *hour = subday;
  subday -= *hour;
  subday *= 60.0;
  *min = subday;
  subday -= *min;
  subday *= 60.0;
  *sec = subday;
}

/****** CALCULATION OF SIDEREAL TIME(rad) ******/
/* mjd: MJD(UT), lo: LONGITUDE of TELESCOPE(rad $BEl7P(B+ $B@>7P(B-) */
double getst(double mjd, double lo){
  double st;
  long i;
  double a1=0.671262;
  double a2=1.002737909;

  st = a1 + a2*(mjd - 40000.0) + lo/6.283185307;
  i = st;
  st = 6.283185307*(st - i);
  return(st);
}

/****** CALCULATION OF HOUR ANGLE(rad) ******/
/* a: SIDEREAL TIME(rad), slo: RA(rad) of the STAR */
double getha(double a, double slo){
  double ha;
  ha = a - slo;
  return(ha);
}

/******* CALCULATION OF ZENITH DISTANCE(rad) ******/
/* d:DEC(rad) of the STAR, phi: LATTITUDE of TELESCOPE(rad $BKL0^(B+ $BFn0^(B-),
   ha: HOUR ANGLE of the star, h: HIGHT(rad) */
double getzd(double d, double phi, double ha){
  double zd, h;
  h = asin(sin(d)*sin(phi) + cos(d)*cos(phi)*cos(ha));
  zd = 1.570796327 - h;
  return(zd);
}

/******* CORRECTION OF PRECESSION *******/
/* $B@V7P(B(a $BC10L(Brad), $B@V0^(B(d $BC10L(Brad)$B$H$=$NJ,E@(B(e), $B5Z$S5a$a$?$$J,E@(B(newe)
   $B$rF~$l$FJd@5$r;\$7$?@V7P(B, $B@V0^$rJV$9!#(B */

double corrra(double a, double d, double e, double newe){
  double da;
  double a1, a2;
  double dt;

  dt = newe - e;
  a1 = 0.000223426676 + 0.000001354812*dt + 5.82e-10*dt*dt;
  a2 = 0.00009718987143 - 0.0000004136915*dt - 1.794e-9*dt*dt;

  da = (a1 + a2*sin(a)*tan(d)) * dt;
  da += a;
  return(da);
}
double corrdec(double a, double d, double e, double newe){
  double dd;
  double a2;
  double dt;

  dt = newe - e;
  a2 = 0.00009718987143 - 0.0000004136915*dt - 1.794e-9*dt*dt;
  dd = a2*cos(d) * dt;
  dd += d;
  a++; /* warning$B$r2sHr$9$k$?$a(B $B!u(B $B>-Mh$N99$J$kJd@5<0:NMQ$N$?$a(B */
  return(dd);
}

/****** CALCULATION OF AIR MASS *****/
/* zd: ZENITH DISTANCE(rad) */
double getfz(double zd){
  double fz;
  double a1=0.0018167;
  double a2=0.002875;
  double a3=0.0008083;

  fz = 1.0/cos(zd) - a1*(1.0/cos(zd) - 1.0) - a2*pow((1.0/cos(zd) - 1.0), 2) - a3*pow((1.0/cos(zd) - 1.0), 3);
  return(fz);
}

/****** Convert to lower characters *****/
char  *chartolower(char *a){
  int i;
  for( i=0; i<=(strlen(a)-1); i++ )
    str4[i] = tolower( a[i] );
  str4[strlen(a)]='\0';
  return( str4 );
}

/****** Convert to upper characters *****/
char  *chartoupper(char *a){
  int i;
  for( i=0; i<=(strlen(a)-1); i++ )
    str4[i] = toupper( a[i] );
  str4[strlen(a)]='\0';
  return( str4 );
}

/****** make error file list *****/
void errlist( char *a, char *b ){
  FILE *fp;
  fp = fopen( "cryofits.log", "at" );
  fprintf( fp, "cannot open '%s'  (%s)\n", a, b );
  fclose( fp );
}

/***************** MAIN ROUTINE ****************/

int main(void){
  time_t t;
  char infile[50], outfile[50];
  FILE *fp1, *fp2, *fp3, *fp4;
  char *listfile="cryolist.dat";
  char *listfile2="CRYOLIST.DAT";
  char buf[100], buftmp[100];
  char thisline[100], lastline[100];
  char buf1[100], buf2[100], buf3[100], buf4[100], buf5[100];
  char buf6[100], buf7[100], buf8[100], buf9[100], buf10[100];
  char buf11[100], buf12[100], buf13[100], buf14[100], buf15[100];
  char buf16[100], buf17[100], buf18[100], buf19[100], buf20[100];
  char buf21[100], buf22[100], buf23[100], buf24[100], buf25[100];
  char buf26[100], buf27[100], buf28[100], buf29[100], buf30[100];
  char buf31[100], buf32[100], buf33[100], buf34[100], buf35[100];
  char buf36[100], buf37[100], buf38[100], buf39[100], buf40[100];
  int i, j;
  int n1, n2, n3;
  double a1, a2;
  double tmpyear, tmpra, tmpdec;
  long l;
  int pix, hb, lb, hborg;
  char pixh[10];

  double ra, dec, equinox;
  float seeing;
  float hw, slitwid;
  char datatype[21], object[21], source[21];
  char weather[21], observer[21], clump[21];
  char weather2[21],cccdtemp[31],cexptime[31];
  char filter[21], diaphrag[21], memo[71];
  char *observatory = OBSERVATORY;
  char *telescope = TELESCOPE;

  double exptime;
  int year, mo, da, ho, mi, se;
  double obsmidtime;
  int pregain, postgain, offset;
  int binfct1, binfct2,binfct3;
  float ccdtemp, nadcspeed, gain;
  char adcspeed[20];
  char detector[30];
  long npix;

  char *objfile="object.lst";
  char *objfile2="OBJECT.LST";
  char tmpobject[21], tmppm[5];
  double tmpequinox;
  int tmprah, tmpram, tmpdecd, tmpdecm;
  double tmpras, tmpdecs;
  int flag_seeing = 0;
  int flag_weather = 0;
  int flag_observer = 0;

  printf( "Edit andor fits file with cryofits4 \n" );
  printf( "  *.fts => *.fit \n" );

  /*--- MAKE HEADER ---*/

  if( ( fp4 = fopen( listfile, "rt" ) ) == NULL ){
    if( ( fp4 = fopen( listfile2, "rt" ) ) == NULL ){
      printf("CANNOT OPEN LIST-FILE (%s)\n", listfile );
      exit(1);
    }
  }

  while( feof( fp4 ) == 0 ){

    fgets( thisline, 100, fp4 );
    if( strcmp( thisline, lastline ) == 0 )
      continue;
    if( strlen( thisline ) < 2 )
      continue;

    sscanf( thisline, "%s %s %s", buf1, buf2, buf3 );
    if( strcmp( buf1, "#" ) == 0 ){
      if( strcmp( buf2, "SEEING" ) == 0 ){
	seeing = atof( buf3 );
	flag_seeing = 1;
      }
      if( strcmp( buf2, "WEATHER" ) == 0 ){
	strcpy( weather, buf3 );
	strcpy( weather2, weather );
	flag_weather = 1;
	//printf("00:%s , flag_weather=%d\n",weather,flag_weather);
      }
      if( strcmp( buf2, "OBSERVER" ) == 0 ){
	if( strlen( buf3 ) <= 28 )
	  strcpy( observer, buf3 );
	else
	  strcpy( observer, mid( buf3, 1, 28 ) );
	flag_observer = 1;
      }
    }
    else{
      if( flag_weather == 0 ){
	do{
	  printf( "WEATHER (1:CLEAR 2:FAIR 3:HAZY 4:CLOUDY) >> " );
	  scanf( "%d", &i );
	}while( i<1 || i>4 );

	switch( i ){
	case 1:
	  strcpy( weather2, "CLEAR" );
	  break;
	case 2:
	  strcpy( weather2, "FAIR" );
	  break;
	case 3:
	  strcpy( weather2, "HAZY" );
	  break;
	case 4:
	  strcpy( weather2, "CLOUDY" );
	}
	strcpy( weather, weather2 );
	flag_weather = 1;
      }
      if( flag_seeing == 0 ){
	printf("SEEING SIZE (arcsec) >> ");
	scanf( "%f", &seeing );
	flag_seeing = 1;
      }
      if( flag_observer == 0 ){
	printf("WHO ARE YOU? >> ");
	scanf( "%s", observer );
	flag_observer = 1;
      }
      strcpy( weather, weather2 );

      sscanf( thisline, "%s %s %d %f %d %f %d %s",\
	      infile, object, &n1, &hw, &n2, &slitwid, &n3, filter );

      strcpy( infile, chartolower( infile ) );

      if ((fp1 = fopen( infile, "rt")) == NULL){
	if( (fp1 = fopen( chartoupper( infile ), "rt" ) ) == NULL ){
	  printf( "Cannot open. (%s)  Skipped\n", infile );
	  errlist( infile, "open" );
	  continue;
	}
      }

      strncpy( outfile, infile, ( strlen( infile ) - 3 ) );
      outfile[ strlen( infile ) - 3 ] = '\0';
      strcat( outfile, "fit" );
      //strcat( outfile, "fits" );
      
      /* move 
      if ((fp2 = fopen(outfile, "wt")) == NULL){
	printf( "Cannot open. (%s)\n", outfile );
	exit(1);
      }
      */

      printf( "%s --> %s  ", infile, outfile );

    SEEKOBJECT:
      if( ( fp3 = fopen( objfile, "rt" ) ) == NULL ){
	if( ( fp3 = fopen( objfile2, "rt") ) == NULL ){
	  printf( "Cannot open (%s)\n", objfile );
	  exit(1);
	}
      }
      while( feof(fp3)==0 ){
	fgets( buf, 100, fp3 );
	sscanf(buf, "%s %lf %2d %2d %lf %1s%2d %2d %lf",\
	       tmpobject, &tmpequinox, &tmprah, &tmpram,\
	       &tmpras, tmppm, &tmpdecd, &tmpdecm, &tmpdecs );
	if( strcmp( tmppm, "+" ) != 0 && strcmp( tmppm, "-" ) != 0 ){
	  printf( "FILE READ ERROR (%s)\n", objfile );
	  exit(1);
	}
	if( strcmp( tmpobject, object ) == 0 ){
	  equinox = tmpequinox;
	  ra = ( (double)tmprah + tmpram/60.0 + tmpras/3600.0 ) * 0.261799387;
	  dec = ( (double)tmpdecd + tmpdecm/60.0 + tmpdecs/3600.0 ) * 0.017453292;
	  if( strcmp( tmppm, "-" ) == 0 )
	    dec *= ( -1.0 );
	  goto PHASE2;
	}
      }
      printf( "CANNOT FIND '%s'.  Skipped\n", object );
      errlist( infile, "OBJECT not find" );
      fclose( fp3 );
      continue;

      PHASE2 :
	fclose( fp3 );

      if( n1<1 || n1>6 ){
	printf("ERROR in DATA-TYPE. (%d) Skipped\n", n1);
	errlist( infile, "DATA-TYPE is not valid" );
	continue;
      }
      switch( n1 ){
      case 1:
	strcpy( datatype, "OBJECT" );
	break;
      case 2:
	strcpy( datatype, "DARK" );
	break;
      case 3:
	strcpy( datatype, "COMPARISON" );
	break;
      case 4:
	strcpy( datatype, "BIAS" );
	break;
      case 5:
	strcpy( datatype, "FLAT" );
	break;
      case 6:
	strcpy( datatype, "NEARBY_SKY" );
	break;
      default:
	strcpy( datatype, "????" );
      }

      if( n2 < 1 || n2 > 9 ){
	printf("ERROR in Diaphragm type. (%d) Skipped\n", n2);
	errlist( infile, "Diaphragm type is not valid" );
	continue;
      }
      switch( n2 ){
      case 1:
	strcpy( diaphrag, "1HOLE" );
	break;
      case 3:
	strcpy( diaphrag, "2HOLES_UPPER" );
	break;
      case 2:
	strcpy( diaphrag, "2HOLES_LOWER" );
	break;
      case 5:
	strcpy( diaphrag, "2SLIT01_UPPER" );
	break;
      case 4:
	strcpy( diaphrag, "2SLIT01_LOWER" );
	break;
      case 7:
        strcpy( diaphrag, "2SLIT02_UPPER" );
        break;
      case 6:
        strcpy( diaphrag, "2SLIT02_LOWER" );
        break;
      case 8:
        strcpy( diaphrag, "1SLIT004" );
      case 9:
        strcpy( diaphrag, "CALIBRAT01" );
        break;
      }

      if( n3 < 1 || n3 > 2 ){
	printf( "ERROR in CLUMP TYPE. (%d) Skipped\n", n3 );
	errlist( infile, "CLUMP type is not valid" );
	continue;
      }
      switch( n3 ){
      case 1:
	strcpy( clump, "WEST" );
	break;
      case 2:
	strcpy( clump, "EAST" );
      }

      if( strcmp( filter, "0" ) == 0 )
      strcpy( filter, "NO_FILTER" );
      if( strcmp( filter, "1" ) == 0 )
	strcpy( filter, "GT" );

      strcpy( source, "CRYOCAM SERVE" );
      strcpy( memo, "");
      binfct1 = 1;
      binfct2 = 1;

      fgets( buf1, 81, fp1 );
      fgets( buf2, 81, fp1 );
      fgets( buf3, 81, fp1 );
      fgets( buf4, 81, fp1 );
      fgets( buf5, 81, fp1 );

      if( atoi( mid( buf4, 27, 4 ) ) == 1000 && \
	 atoi( mid( buf5, 27, 4 ) ) == 1018 ){
	printf( "(T5 CRYOCAM mode)\n" );
	sprintf( detector, "T5 CRYOCAM (TC215)" );
	npix = 1000L * 1018L;

	fgets( buf6, 81, fp1 );
	fgets( buf7, 81, fp1 );

	/* Select header type */
	fgets( buf, 81, fp1 );
	if( strcmp( mid( buf, 1, 8 ), "BIN-FCT1" ) != 0 ){
	  /* Version A */
	  sscanf( buf, "%s %s %lf %s", buftmp, buftmp, &exptime, buftmp );

	  fgets(  buf, 14, fp1 );
	  fgets(  buf, 3, fp1 );
	  da = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mo = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 5, fp1 );
	  year = atoi( buf );
	  fgets(  buf, 4, fp1 );
	  ho = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mi = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  se = atoi( buf );
	  fgets(  buf, 49, fp1 );

	  binfct1 = 1;
	  binfct2 = 1;
	  fgets( buf, 81, fp1 );
	  fgets( buf, 81, fp1 );

	  fgets( buf, 81, fp1 );
	}
	else{
	  /* Version B */
	  sscanf( buf, "%s %s %d %s", buftmp, buftmp, &binfct1, buftmp );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %d %s", buftmp, buftmp, &binfct2, buftmp );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %lf %s", buftmp, buftmp, &exptime, buftmp );

	  fgets(  buf, 13, fp1 );
	  fgets(  buf, 3, fp1 );
	  da = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mo = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 5, fp1 );
	  year = atoi( buf );
	  fgets(  buf, 4, fp1 );
	  ho = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mi = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  se = atoi( buf );
	  fgets(  buf, 50, fp1 );

	  
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s", buftmp, buftmp, buftmp );
	  if( strcmp( buftmp, "BINNING" ) == 0 ){
	  /* Version C */
	    fgets( buf, 81, fp1 );
	    fgets( buf, 81, fp1 );
	  }
	}

	sscanf( buf, "%s %s %s %s %s %s %d", buftmp, buftmp, buftmp,\
	       buftmp, buftmp, buftmp, &pregain );
	fgets( buf, 81, fp1 );
	sscanf( buf, "%s %s %s %s %s %s %d", buftmp, buftmp, buftmp,\
	       buftmp, buftmp, buftmp, &postgain );
	fgets( buf, 81, fp1 );
	sscanf( buf, "%s %s %s %s %s %d", buftmp, buftmp, buftmp,\
	       buftmp, buftmp, &offset );

	fgets(  buf, 81, fp1 );

	fgets( buf, 81, fp1 );
	sscanf( buf, "%s %s %s %s %s %s", buftmp, buftmp, buftmp,\
	       buftmp, buftmp, adcspeed );

	fgets( buf, 81, fp1 );
	sscanf( buf, "%s %s %s %s %s %s %f", buftmp, buftmp, buftmp,\
	       buftmp, buftmp, buftmp, &ccdtemp );
      }
      else if( atoi( mid( buf4, 27, 4 ) ) == 512 && \
	   atoi( mid( buf5, 27, 4 ) ) == 512 ){
	  printf( "(S5 CRYOCAM mode)\n" );
	  sprintf( detector, "S5 CRYOCAM (SITe)" );
	  npix = 512L * 512L;

	  fgets( buf6, 81, fp1 );
	  fgets( buf7, 81, fp1 );

	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %d %s", buftmp, buftmp, &binfct1, buftmp );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %d %s", buftmp, buftmp, &binfct2, buftmp );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %lf %s", buftmp, buftmp, &exptime, buftmp );

	  fgets(  buf, 13, fp1 );
	  fgets(  buf, 3, fp1 );
	  da = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mo = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 5, fp1 );
	  year = atoi( buf );
	  fgets(  buf, 4, fp1 );
	  ho = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  mi = atoi( buf );
	  fgets(  buf, 2, fp1 );
	  fgets(  buf, 3, fp1 );
	  se = atoi( buf );
	  fgets(  buf, 50, fp1 );

	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s %s %s %s %d", buftmp, buftmp, buftmp,\
		 buftmp, buftmp, buftmp, &pregain );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s %s %s %s %d", buftmp, buftmp, buftmp,\
		 buftmp, buftmp, buftmp, &postgain );
	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s %s %s %d", buftmp, buftmp, buftmp,\
		 buftmp, buftmp, &offset );

	  fgets(  buf, 81, fp1 );

	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s %s %s %s", buftmp, buftmp, buftmp,\
		 buftmp, buftmp, adcspeed );

	  fgets( buf, 81, fp1 );
	  sscanf( buf, "%s %s %s %s %s %s %f", buftmp, buftmp, buftmp,\
		 buftmp, buftmp, buftmp, &ccdtemp );

	}
      else if( atoi( mid( buf4, 27, 4 ) ) == 2048 && \
	   atoi( mid( buf5, 27, 4 ) ) == 512 ) // add for AndorCCD on 2005 08/24
	{
	  printf( "(Andor CCD mode)\n" );
	  sprintf( detector, "Andor DV440" );
	  npix = 2048L * 512L;

	  fgets( buf6, 81, fp1 );
	  fgets( buf7, 81, fp1 );
	  fgets( buf8, 81, fp1 );
	  fgets( buf9, 81, fp1 );
	  fgets( buf10, 81, fp1 );
	  fgets( buf11, 81, fp1 );
	  fgets( buf12, 81, fp1 );
	  fgets( buf13, 81, fp1 );
	  fgets( buf14, 81, fp1 );
	  fgets( buf15, 81, fp1 );
	  fgets( buf16, 81, fp1 );
	  fgets( buf17, 81, fp1 );
	  fgets( buf18, 81, fp1 );
	  fgets( buf19, 81, fp1 );
	  fgets( buf20, 81, fp1 );
	  fgets( buf21, 81, fp1 );
	  fgets( buf22, 81, fp1 );
	  fgets( buf23, 81, fp1 );
	  fgets( buf24, 81, fp1 );
	  fgets( buf25, 81, fp1 );
	  fgets( buf26, 81, fp1 );
	  fgets( buf27, 81, fp1 );
	  fgets( buf28, 81, fp1 );
	  fgets( buf29, 81, fp1 );
	  fgets( buf30, 81, fp1 );
	  /*
	  fgets( buf31, 81, fp1 );
	  fgets( buf32, 81, fp1 );
	  fgets( buf33, 81, fp1 );
	  fgets( buf34, 81, fp1 );
	  fgets( buf35, 81, fp1 );
	  fgets( buf36, 81, fp1 );
	  fgets( buf37, 81, fp1 );
	  fgets( buf38, 81, fp1 );
	  fgets( buf39, 81, fp1 );
	  fgets( buf40, 81, fp1 );
	  */

	} // add for AndorCCD on 2005 08/24
	else{
	  printf( "FITS NAXIS values are strange... Skip.\n" );
	  errlist( infile, "FITS NAXIS values are strange..." );
	  continue;
	}
      fclose( fp1 );

      obsmidtime = timetomjd( year, mo, da, ho, mi, se ) + 5.787037e-6 * exptime - 0.375;
      mjdtotime(obsmidtime, &year, &mo, &da, &ho, &mi, &se);
      tmpyear = year + (obsmidtime - timetomjd(year, 1, 1, 0, 0, 0))/365.25;
      tmpra = corrra(ra, dec, equinox, tmpyear);
      tmpdec = corrdec(ra, dec, equinox, tmpyear);

      printf("%10s (RA=%s DEC=%s %6.1f) %10s  %04d-%02d-%02d %02d:%02d:%02d\n",\
	     object, fitsra( ra ), fitsdec( dec ), equinox, datatype,\
	     year, mo, da, ho, mi, se );

      if ((fp2 = fopen(outfile, "wt")) == NULL){
	printf( "Cannot open. (%s)\n", outfile );
	exit(1);
      }

      if( atoi( mid( buf4, 27, 4 ) ) != 2048 && \
	  atoi( mid( buf5, 27, 4 ) ) != 512 ){
      }
      else
	{
	  // for AndorCCD
	  for(i=1; i<=36; i++){
	    
	    switch(i){
	    case 1 :
	      strcpy( buf, buf1 );
	      break;
	    case 2 :
	      strcpy( buf, buf2 );
	      break;
	    case 3 :
	      strcpy( buf, buf3 );
	      break;
	    case 4 :
	      strcpy( buf, buf4 );
	      break;
	    case 5 :
	      strcpy( buf, buf5 );
	      break;
	    case 6 :
	      strcpy( buf, buf6 );
	      break;
	    case 7 :
	      strcpy( buf, buf7 );
	      break;
	    case 8 :
	      strcpy( buf, buf8 );
	      break;
	    case 9 :
	      strcpy( buf, buf9 );
	      break;
	    case 10 :
	      strcpy( buf, buf10 );
	      break;
	    case 11 :
	      strcpy( buf, "BZERO   =                32768 / offset data range to that of unsigned short");
	      //strcpy( buf, "BZERO   =                    0 / offset data range to that of unsigned short");
	      break;
	    case 12 :
	      strcpy( buf, "BSCALE  =                    1 / default scaling factor" );
	      break;
	    case 13 :
	      strcpy( buf, buf11 );
	      break;
	    case 14 :
	      strcpy( buf, buf12 );
	      break;
	    case 15 :
	      strcpy( buf, buf13 );
	      break;
	    case 16	:
	      strcpy( buf, buf14 );
	      break;
	    case 17	:
	      strcpy( buf, buf15 );
	      break;
	    case 18	:
	      strcpy( buf, buf16 );
	      break;
	    case 19	:
	      strcpy( buf, buf17 );
	      break;
	    case 20	:
	      strcpy( buf, buf18 );
	      break;
	    case 21	:
	      strcpy( buf, buf19 );
	      break;
	    case 22	:
	      strcpy( buf, buf20 );
	      break;
	    case 23	:
	      strcpy( buf, buf21 );
	      break;
	    case 24	:
	      strcpy( buf, buf22 );
	      break;
	    case 25	:
	      strcpy( buf, buf23 );
	      break;
	    case 26	:
	      strcpy( buf, buf24 );
	      break;
	    case 27	:
	      strcpy( buf, buf25 );
	      break;
	    case 28	:
	      strcpy( buf, buf26 );
	      break;
	    case 29	:
	      strcpy( buf, buf27 );
	      break;
	    case 30	:
	      strcpy( buf, buf28 );
	      break;
	    case 31	:
	      strcpy( buf, buf29 );
	      break;
	    case 32	:
	      strcpy( buf, buf30 );
	      break;
	    case 33	:
	      strcpy( buf, buf30 );
	      break;
	    case 34	:
	      strcpy( buf, buf30 );
	      break;
	    case 35	:
	      strcpy( buf, buf30 );
	      break;
	    case 36	:
	      strcpy( buf, buf30 );
	      break;
	    case 37	:
	      strcpy( buf, buf30 );
	      break;
	    case 38	:
	      strcpy( buf, buf30 );
	      break;
	    case 39	:
	      strcpy( buf, buf30 );
	      break;
	    case 40	:
	      strcpy( buf, buf30 );
	      break;
	    }
	    fprintf( fp2,"%-80s", buf );
	  }
	}

     fclose( fp2 );

      /*--- CONVERT AND CREATE BINARY DATA ---*/

      if ((fp1 = fopen(infile, "rb")) == NULL){
	if( (fp1 = fopen( chartoupper( infile ), "rt" ) ) == NULL ){
	  printf( "Cannot open. (%s)\n", infile );
	  exit(1);
	}
      }

      for( j=1; j<=2880*1; j++ )
	i = getc( fp1 );

      if ((fp2 = fopen(outfile, "ab")) == NULL){
	printf( "Cannot open. (%s)\n", outfile );
	exit(1);
      }

      for( l=1L; l<=npix; l++ ){
	hb = getc( fp1 );
	lb = getc( fp1 );

	hborg = hb;
	if (hb >= 128 )
	  {
	    hb = hb-128;
	  }
	else
	  {
	    hb=hb+128;
	  }
	/*      putc( 0, fp2 );
		putc( 0, fp2 ); */
	//printf("%7d: %d %d  %d\n",l,hborg,hb,lb);
	putc( hb, fp2 );
	putc( lb, fp2 );
      }
      if( ( (npix*2L) % 2880L ) != 0 ){
	for( l=1L; l<=( 2880L - (npix*2L) % 2880L); l++ )
	  putc( 0, fp2 );
      }

      fclose( fp1 );
      fclose( fp2 );

      strcpy( lastline, thisline );
    }
  }
  
  fclose( fp4 );
  printf("---- FINISHED ----\n");

   return( 0 );
}
